// PlexStimExample2.cpp : Defines the entry point for the console application.
//
// Arbitrary Pattern Example
// Loads an arbitrary stimulation pattern from
// the sample file: 
// configures channel 1 of stimulator 1
// and stimulates until the user exits the program

#include <iostream>
#include "PlexStim.h"

// An error reporting function
// used to save some typing in the main program
// Does nothing if no error, reports the error if there is one
void PS_Error_Report(char* function_name, int error_code)
	{	
		if PS_Failed(error_code)
		{	
			char error_desc[128] = {'\0'};
			PS_GetExtendedErrorInfo(error_code, error_desc);
			printf("%s returned %s\n", function_name, error_desc);
		}
	}


int main()
{
// the name of the stimulation pattern file to open
char filename[] = "c:\\PlexonData\\Stim-2\\Waveform pattern files\\spike_160k_20uA.pat";
// char filename[] = "c:\\PlexonData\\Stim-2\\TestFiles\\Mismatched_variable.pat";

// define some variables
int res = 0;					// the result of a function call (error code)
char error_desc[128] = {'\0'};	// an English language description of the error code
int NStim = 0;					// the number of stimulators connected
char serial[16] = {'\0'};		// the serial number
char description[64] = {'\0'};	// the "description" == the hardware model

int stim = 1;					// the example uses stimulator #1
int chan = 1;					// the example uses channel #1
int rate = 100;					// the example stimulates at a rate of 100 Hz
//int period = 10;				// the example stimulates with a period of 10 ms 
int reps = 0;					// repetitions, 0 stands for infinite repetitions


// Initialize all of the stimulators that are connected (and turned on)
// PS_InitAllStim must be called first
// An error will result if no stimulators are detected
res = PS_InitAllStim();
PS_Error_Report("PS_InitAllStim", res);

// Display the number of stimulators that were detected
res = PS_GetNStim(&NStim);
PS_Error_Report("PS_GetNStim", res);
printf("PS_GetNStim: %.1d stimulators were detected\n\n", NStim);

// Do the following only if there are stimulators connected.
if(NStim > 0)
{
	// Optional: retrieve basic information about each stimulator
	// Note that the stimulators enumerate in serial number order
	// Stimulator #1 is the lowest number serial number
	for(int i=1; i<=NStim; i+=1)
	{	
		// Read and display the Serial Number
		res = PS_GetSerialNumber(i, serial);
		PS_Error_Report("PS_GetSerialNumber", res);
		if (res==0) printf("PS_GetSerialNumber, stimulator %.1d = %s\n", i, serial);
		
		// Read and display the Description == Hardware version
		res = PS_GetDescription(i, description);
		PS_Error_Report("PS_GetDescription", res);
		if (res==0) printf("PS_GetDescription , stimulator %.1d = %s\n", i, description);
	}

	// Display information about the selected stimulator and channel
	printf("\n");
	printf("Working with stimulator %.1d \n", stim);
	printf("Programming channel %.1d \n", chan);
	printf("pattern file: %s\n", filename);

	// Configure the stimulator
	// Set the monitor channel
	res = PS_SetMonitorChannel(stim, chan); 
	PS_Error_Report("PS_SetMonitorChannel", res);

	// Set the pattern type for the channel (default is rectangular )
	res = PS_SetPatternType(stim, chan, PS_PATTERN_ARB);
	PS_Error_Report("PS_SetPatternType", res);

	// load the stimulation pattern from one of the example files
	res = PS_LoadArbPattern(stim, chan, filename);
	PS_Error_Report("PS_LoadArbPattern", res);

	// Set the stimulation rate or period
	res = PS_SetRate(stim, chan, rate);				// Set the rate in Hz
	PS_Error_Report("PS_SetRate", res);
	//res = PS_SetPeriod(stim, chan, period);		// or set the period in ms
	
	// set the number of repetitions
	res = PS_SetRepetitions(stim, chan, reps);  
	PS_Error_Report("PS_SetRepetitions", res);

	// set the "trigger mode" to decide what inputs cause stimulation
	res = PS_SetTriggerMode(stim, PS_TRIG_SOFT);		// only a software command (your program)
	//PS_SetTriggerMode(stim, PS_TRIG_PULSE);			// stimulation is started by a digital input
	//PS_SetTriggerMode(stim, PS_TRIG_LEVEL);			// stimulation is started by a digital input
	PS_Error_Report("PS_SetTriggerMode", res);

	// download the configuration information to the stimulator hardware
	res = PS_LoadChannel(stim, chan);
	PS_Error_Report("PS_LoadChannel", res);

	// Trigger the stimulation
	res = PS_StartStimChannel(stim, chan);
	printf("Starting stimulation, channel %.1d\n", chan);
	PS_Error_Report("PS_StartStimChannel", res);

}// if NStim > 0

// wait for user input to exit
printf("\n");
printf("Hit enter to stop stimulation and exit\n");
//std::gets(error_desc);
std::getchar();

// stop the stimulation
res = PS_StopStimChannel(stim, chan);
PS_Error_Report("PS_StopStimChannel", res);

// You must call PS_CloseStim or PS_CloseAllStim to release the stimulator
// before another program can use it
res = PS_CloseAllStim();
PS_Error_Report("PS_CloseAllStim", res);

// exit the program
return 0;
}

