////////////////////////////////////////
//                                    //
//    Plexon ElectroStim API          //
//                                    //
////////////////////////////////////////
//                                    //
//    Copyright 2015 Plexon Inc       //
//    Dallas Texas USA                //
//                                    //
//    Ver. 2.2.0.0  July 2014  TVM    //
//    Ver. 2.3.17.0	April 2015 A.E.   //
//                                    //
////////////////////////////////////////

// NOTE:
// Requires PlexStimTypes.h
//
// This API supports Plexon electrostimulators only 
// This API supports up to FOUR stimulators.


// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the PLEXSTIM_EXPORTS
// symbol defined on the command line. This symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// PLEXSTIM_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef PLEXSTIM_EXPORTS
#define PLEXSTIM_API __declspec(dllexport)
#else
#define PLEXSTIM_API __declspec(dllimport)
#endif


#include "PlexStimTypes.h"


#define PS_Failed(error) ((error) != 0)

// PS_GetExtendedErrorInfo - Returns an English language description of the specified error code
//In:
//		*ErrorString	- pointer to a character array to store the error code description
//		ErrorCode		- the error code for which you want a description
//Returns:
//		0	- OK
//		14	- character array is not initialized (the pointer is null)
// Effect:
//		ErrorString is updated with a brief English language description of the specified error code.
//
// Note:
//		A buffer of at least 64 bytes must be allocated before calling this function
//		example C++ code:
//		char errBuff[128] = {'\0'};
//		int dwsStimN = 0;
//		res = PS_GetNStim(&dwsStimN);
//		if(PS_Failed(res)){
//			PS_GetExtendedErrorInfo(res, errBuff);
//			std::cout << errBuff;
//		}
//
PLEXSTIM_API int      PS_GetExtendedErrorInfo(int ErrorCode, char *ErrorString);


// PS_InitStim - Initialize all available Plexon electrostimulators
//Returns:
//		0 - OK
//		1 - error initializing devices
//		2 - no Plexon stimulators found
// Effect:
//		Stimulators are initialized and placed in stimulation mode (versus Z test mode).
//
PLEXSTIM_API int       PS_InitAllStim(void);


// PS_GetNStim - Get number of available Plexon Stimulators.
//In:
//		*NumStim	- pointer to variable to store the number of stimulators.
// Returns:
//		0 - OK
//		NumStim is updated with the number of avilable Plexon Stimulators
// Effect:
//      Gets number of available Plexon Stimulators.
// Note:
//		The maximum number of stimulators you can work with is four.
//
PLEXSTIM_API int       PS_GetNStim(int *NumStim);


// PS_CloseStim - Finalize work with specified stimulator
// In: 
//		StimN  - stimulator number to finalize (starts from 1)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
// Effect:
//      Finalizes work with specified stimulator. Any stimulation in progress is aborted.
//
PLEXSTIM_API int       PS_CloseStim(int StimN);


// PS_CloseAllStim - Finalize work with all available stimulators
// Returns:
//		0 - OK
//		1 - device error
// Effect:
//      Finalizes work with all available stimulators. Any stimulation in progress is aborted.
//
PLEXSTIM_API int       PS_CloseAllStim(void);


// PS_Abort - Cease all stimulation for the specified stimulator immediately (emergency)
// In: 
//      StimN  - stimulator number to abort (starts from 1)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
// Effect:
//      Causes all stimulation to cease immediately even if there is a pulse or arbitrary waveform in progress. 
//		This is in contrast to stopping stimulation by calling PS_StopStimChannel or PS_StopStimAllChannels.
//
PLEXSTIM_API int       PS_Abort(int StimN);


// PS_AbortAll - Cease all stimulation for all available electrostimulators immediately
// Returns:
//		0 - OK
//		1 - device error
// Effect:
//      Causes all stimulation to cease immediately even if there is a pulse or arbitrary waveform in progress. 
//		This is in contrast to stopping stimulation by calling PS_StopStimChannel or PS_StopStimAllChannels.
//
PLEXSTIM_API int       PS_AbortAll(void);


// PS_GetNChannels - Get maximum number of channels
// In: 
//      StimN	- stimulator number to query (starts from 1)
//		*ChN	- pointer to variable to store number of channels
// Returns: 
//		-1 invalid argument(s)
//		 0 - OK
//		 ChN is updated with the maximum number of channels; 
// Effect:
//      Returns maximum number of channels for the stimulator StimN.
//
PLEXSTIM_API int       PS_GetNChannels(int StimN, int *ChN);


// PS_LoadChannel - Load stimulation parameters for specified channel to stimulator hardware
// In: 
//		StimN  - stimulator number to configure (starts from 1)
//		ChN  - channel number which parameters will be loaded in the stimulator hardware (starts from 1)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
//		 3 - CRC Error
//		 6 - stimulation pattern is not ready (in case of loading an arbitrary pattern)
// Effect:
//		Loads stimulation parameters for specified channel to the stimulator hardware.
//
PLEXSTIM_API int       PS_LoadChannel(int StimN, int ChN);


// PS_LoadAllChannels - Load stimulation parameters for all channels to stimulator hardware
// In: 
//		StimN  - stimulator number to configure (starts from 1)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
//		 3 - CRC Error
//		 6 - stimulation pattern(s) is(are) not ready (in case of loading an arbitrary pattern)
// Effect:
//		Loads stimulation parameters for all channels to stimulator hardware.
//
PLEXSTIM_API int       PS_LoadAllChannels(int StimN);


// PS_StartStimChannel - Start stimulation for specified channel
// In: 
//		StimN  - stimulator number to configure (starts from 1)
//		ChN  - channel number to start stimulation (starts from 1)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
//		 4 - wrong trigger mode (trigger mode doesn't set to PS_TRIG_SOFT)
// Effect:
//		Start stimulation for specified channel (with previously configured parameters).
//
PLEXSTIM_API int       PS_StartStimChannel(int StimN, int ChN);


// PS_StopStimChannel - Stop stimulation for specified channel
// In: 
//		StimN  - stimulator number to configure (starts from 1)
//		ChN  - channel number to stop stimulation (starts from 1)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
//		 4 - wrong trigger mode (trigger mode doesn't set to PS_TRIG_SOFT)
// Effect:
//		Stops stimulation for specified channel.
//
PLEXSTIM_API int       PS_StopStimChannel(int StimN, int ChN);


// PS_ChannelStimStarted - Check if stimulation was started for specified channel
// In: 
//      StimN		- stimulator number to query (starts from 1)
//      ChN			- channel number to query (starts from 1)
//		*bStarted	- pointer to variable to store started status
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		bStarted = TRUE if stimulation started for channel ChN, FALSE otherwise
// Effect:
//		Checks if stimulation was started for the specified channel.
//
PLEXSTIM_API int       PS_ChannelStimStarted(int StimN, int ChN, bool *bStarted);


// PS_StartStimAllChannels - Start stimulation for all channels
// In: 
//      StimN  - stimulator number to configure (starts from 1)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
//		 4 - wrong trigger mode (trigger mode isn't set to PS_TRIG_SOFT)
// Effect:
//		Starts stimulation on all channels for the specified stimulator.
//
PLEXSTIM_API int       PS_StartStimAllChannels(int StimN);


// PS_StopStimAllChannels - Stop stimulation for all channels
// In: 
//		StimN  - stimulator number to configure (starts from 1)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
//		 4 - wrong trigger mode (trigger mode isn't set to PS_TRIG_SOFT)
// Effect:
//		Stops stimulation on all channels for the specified stimulator.
//
PLEXSTIM_API int       PS_StopStimAllChannels(int StimN);


// PS_SetMonitorChannel - Select the channel to monitor
// In: 
//		StimN  - stimulator number to configure (starts from 1)
//		ChN  - channel to monitor (starts from 1)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
// Effect:
//      Configures the voltage and current monitor outputs to select
//		the specified channel for monitoring.
//
PLEXSTIM_API int       PS_SetMonitorChannel(int StimN, int ChN);


// PS_GetMonitorChannel - Get channel being monitored
// In: 
//		StimN	- stimulator number to query (starts from 1)
//		*MonChN	- pointer to variable to store monitor channel
// Returns: 
//		-1 - invalid argument(s)
//		 0 - OK
//		 MonChN is updated with the channel that is being monitored
// Effect:
//      Gets the channel that is selected for monitoring.
//
PLEXSTIM_API int       PS_GetMonitorChannel(int StimN, int *MonChN);


// PS_SetVmonScaling - Set the scale factor for the voltage monitor output
// In: 
//		StimN	- stimulator number to configure (starts from 1)
//		sc		- scale factor for voltage monitor output (one of PS_VMON_SCALING choices, defined in PlexStimTypes.h)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
// Effect:
//		Selects the scale factor for the voltage monitor output.
//		Default value is PS_VMON_SCALING_0_25 (0.25 V/V)
//
PLEXSTIM_API int       PS_SetVmonScaling(int StimN, PS_VMON_SCALING sc);


// PS_GetVmonScaling - Get the scale factor for the voltage monitor output
// In: 
//		StimN			- stimulator number to query (starts from 1)
//		*vmon_scaling	- pointer to variable to store the scale factor (type PS_VMON_SCALING)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		vmon_scaling will be updated with the current voltage monitor scale factor (type PS_VMON_SCALING, defined in PlexStimTypes.h)
// Effect:
//		Gets the current voltage monitor scale factor.
//		Default value is PS_VMON_SCALING_0_25 (0.25 V/V)
//
PLEXSTIM_API int PS_GetVmonScaling(int StimN, PS_VMON_SCALING *vmon_scaling);


// PS_SetPatternType -	Configures selected channel for bi-phasic rectangular pulse or 
//						arbitrary waveform pattern operation
// In: 
//		StimN - stimulator number to configure (starts from 1)
//		ChN   - channel number to configure (starts from 1) 
//		type  - pattern type to use (PS_PATTERN_TYPE defined in PlexStimTypes.h)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
// Effect:
//		Configures the selected channel to use a bi-phasic rectangular pulse (defined using PS_SetRectParam) or 
//		an arbitrary waveform pattern (loaded from a file using PS_LoadArbPattern).
//		Default value is PS_PATTERN_RECT.
//
PLEXSTIM_API int       PS_SetPatternType(int StimN, int ChN, PS_PATTERN_TYPE type);


// PS_GetPatternType -	Checks the configuration of the specified channel
// In: 
//		StimN - stimulator number to query (starts from 1)
//		ChN   - channel number to query (starts from 1)
// Returns:
//		-1 if invalid argument(s)
//		 0 - OK
//		type is updated with the pattern type of the specified channel (PS_PATTERN_TYPE defined in PlexStimTypes.h)
// Effect:
//		Checks if the specified channel is configured for bi-phasic rectangular pulse (PS_PATTERN_RECT) or 
//		arbitrary waveform pattern (PS_PATTERN_ARB) operation.
//
PLEXSTIM_API int       PS_GetPatternType(int StimN, int ChN, PS_PATTERN_TYPE *type);


// PS_LoadArbPattern - Loads waveform information from a waveform pattern (.pat) file
// In: 
//		StimN - stimulator number to configure (starts from 1)
//		ChN   - channel number to configure (starts from 1)
//		*s    - full path (not more than 512 characters) of the file containing arbitrary waveform pattern
//Returns:
//		-1 - invalid argument(s)
//		 0 - pattern loaded from file successfully
//		 7 - length of the file name exceeds 512
//		 8 - file doesn't exist
//		 9 - file is opened by another process
//		 10 - file contains too many lines (max is 1002)
//		 11 - file contains invalid value(s)
//		 12 - file contains too few lines (min is 3)
//		 13 - file contains a mismatched amplitude duration pair 
// Effect:
//		Loads waveform information for the specified channel from an arbitrary waveform pattern (.pat) file.
//
PLEXSTIM_API int       PS_LoadArbPattern(int StimN, int ChN, char* s);


// PS_GetNPointsArbPattern - Get the number of points in a graphical representation of the arbitrary waveform
// In: 
//		StimN	- stimulator number to query (starts from 1)
//		ChN		- channel number to query (starts from 1)
//Returns:
//		-1		- invalid argument(s)
//		 0		- OK
//		points	- pointer to a variable to store the number of points
// Effect:
//		Returns the number of points in a graphical representation of the arbitrary waveform 
//		pattern loaded into the selected stimulator and channel
//
PLEXSTIM_API int       PS_GetNPointsArbPattern(int StimN, int ChN, int *points);


// PS_GetArbPatternPoints - Get X and Y coordinates of a grapical representation of 
//							the arbitrary waveform loaded for the channel
// In: 
//      StimN  - stimulator number to query (starts from 1)
//      ChN  - channel number to query (starts from 1)
//		NPoints - number of points
//		*Coords - pointer to an array to store the coordinates, the size of the array should be >= 2 * NPoints 
//				  Coords contains coordinates as X-Y pairs: x1 y1 x2 y2 ....
//Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 5 - NPoints is not equal to the number of point (See note below)
// Effect:
//		Returns the X and Y coordinates of a graphical representation of the arbitrary waveform 
//		pattern loaded into the selected stimulator and channel.
//		These coordinates can be used to draw a graph of the pattern
// Note:
//		Call PS_GetNPointsArbPattern to get NPoints before calling this function
//
PLEXSTIM_API int PS_GetArbPatternPoints(int StimN, int ChN, int NPoints, int* Coords);


// PS_GetArbPatternPointsX - Get X coordinates of a graphical representation of 
//							 the arbitrary waveform loaded for the channel
// In: 
//      StimN	 - stimulator number to configure (starts from 1)
//      ChN		 - channel number to configure (starts from 1)
//		NPoints  - number of points
//		*XCoords - pointer to an array to store the coordinates, the size of the array should be >= NPoints 
//					XCoords contains X coordinates as x1 x2 x3 ....
//Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
//		 5 - NPoints is not equal number of point for the pattern in this channel
// Effect:
//		Returns the X coordinates only of a graphical representation of the arbitrary waveform 
//		pattern loaded into the selected stimulator and channel.
//		These coordinates can be used to draw a graph of the pattern
// Note:
//		Call PS_GetNPointsArbPattern to get NPoints before calling this function
//
PLEXSTIM_API int PS_GetArbPatternPointsX(int StimN, int ChN, int NPoints, int* XCoords);


// PS_GetArbPatternPointsY - Get Y coordinates of a graphical representation of 
//							 the arbitrary waveform loaded for the channel
// In: 
//      StimN	 - stimulator number to query (starts from 1)
//      ChN		 - channel number to query (starts from 1)
//		NPoints  - number of points
//		*YCoords - pointer to an array to store the coordinates, the size of the array should be >= NPoints 
//					YCoords contains Y coordinates as y1 y2 y3 ....
//Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
//		 5 - NPoints is not equal number of point for the pattern in this channel
// Effect:
//		Returns the Y coordinates only of a graphical representation of the arbitrary waveform 
//		pattern loaded into the selected stimulator and channel.
//		These coordinates can be used to draw a graph of the pattern
// Note:
//		Call PS_GetNPointsArbPattern to get NPoints before calling this function
//
PLEXSTIM_API int PS_GetArbPatternPointsY(int StimN, int ChN, int NPoints, int* YCoords);


// PS_SetRectParam - Set the bi-phasic rectangular pulse parameters for the specified channel
// In: 
//		StimN	- stimulator number to configure (starts from 1)
//		ChN		- channel number to configure (starts from 1)
//		Param	- PS_RectPattern structure containing parameters of the rectangular pulse; defined in PlexStimTypes.h
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
// Effect:
//		Sets the bi-phasic rectangular pulse parameters for the specified channel
//		the default values are:
//		- first phase amplitude = 100 mA;
//		- second phase amplitude = -100 mA;
//		- first phase width = 50 us;
//		- second phase width = 50 us;
//		- interphase delay = 25 us;
//
PLEXSTIM_API int       PS_SetRectParam(int StimN, int ChN, PS_RectPattern Param);


// PS_GetRectParam - Get the bi-phasic rectangular pulse parameters for the specified channel
// In: 
//		StimN	 - stimulator number to query (starts from 1)
//		ChN		 - channel number to query (starts from 1)
//		*pattern - pointer to a structure to store the pulse parameters (defined in PlexStimTypes.h)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
//		 pattern is updated with the bi-phasic pulse parameters for the specified channels
// Effect:
//		Gets the bi-phasic pulse parameters for the specified channels
//
PLEXSTIM_API int       PS_GetRectParam(int StimN, int ChN, PS_RectPattern *pattern);


// PS_SetRepetitions - Set number of repetitions for the specified channel
// In: 
//		StimN - stimulator number to configure (starts from 1)
//		ChN   - channel number to configure (starts from 1)
//		R     - number of repetitions, can range from 1 to 32767; use 0 for an infinite number of repetitions.
// Returns:
//     -1 - invalid argument(s)
//		0 - OK;
// Effect:
//      Sets the number of times that the bi-phasic pulse (defined using PS_SetRectParam) 
//		or the arbitrary waveform (loaded from a text file using PS_LoadArbPattern) is 
//		repeated for the specified channel. Default value is 1.
//
PLEXSTIM_API int       PS_SetRepetitions(int StimN, int ChN, int R);


// PS_GetRepetitions - Get number of repetitions for specified channel
// In: 
//		StimN   - stimulator number to query (starts from 1)
//		ChN     - channel number to query (starts from 1)
//		*R		- pointer to a variable to store the number of repetitions
//Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
//		 R is updated with the number of repetitions
// Effect:
//		Gets the number of repetitions, the number of times that the rectangular pulse 
//		or arbitrary waveform is repeated.
//		Default value is 1.
//
PLEXSTIM_API int       PS_GetRepetitions(int StimN, int ChN, int *R);


// PS_SetRate - Set the repetition rate for the specified channel (in Hertz)
// In: 
//      StimN  - stimulator number to configure (starts from 1)
//      ChN  - channel number to configure (starts from 1)
//      rate - rate in Hertz; valid range: 0.008 Hz <= rate <= 50000 Hz
//Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
// Effect:
//		Sets the rate at which the bi-phasic pulse or arbitrary waveform is repeated for the specified channel
//		Default value is 200 Hz.
// Note:
//		If the requested rate is faster than the requested pulse/waveform can be repeated (because of its duration)
//		the actual rate will be coerced to accomodate the length of the pulse/waveform
//
PLEXSTIM_API int       PS_SetRate(int StimN, int ChN, double rate);


// PS_GetRate - Get the repetition rate for the specified channel (in Hertz)
// In: 
//      StimN	- stimulator number to query (starts from 1)
//      ChN		- channel number to query (starts from 1)
//		*rate	- pointer to a variable to store the rate
//Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
//		rate is updated with the repetition rate (in Hertz)
// Effect:
//      Gets the repetition rate (in Hertz) for the specified channel
//		Default value is 200 Hz.
// Note:
//		The requested rate, not the coerced rate, is returned
//		See PS_SetRate and PS_GetStimPatternDuration
//
PLEXSTIM_API int       PS_GetRate(int StimN, int ChN, double *rate);


// PS_SetPeriod - Set the repetition period for the specified channel (in milliseconds)
// In: 
//      StimN	- stimulator number to configure (starts from 1)
//      ChN		- channel number to configure (starts from 1)
//      period	- period value in milliseconds; valid range: 0.020 ms <= period <= 125,000 ms
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
// Effect:
//		Sets the repetition period (in milliseconds) for the specified channel
//		Default value is 5 ms.
// Note:
//		If the requested period is shorter than the requested pulse/waveform duration
//		the actual period will be coerced to accomodate the pulse/waveform duration
//
PLEXSTIM_API int       PS_SetPeriod(int StimN, int ChN, double period);


// PS_GetPeriod - Get the repetition period for the specified channel (in milliseconds)
// In: 
//		StimN	- stimulator number to query (starts from 1)
//      ChN		- channel number to query (starts from 1) 
//		*period	- pointer to a variable to store the period
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
//		period is updated with the repetition period (in milliseconds) for the specified channel 
// Effect:
//      Gets repetition period (in milliseconds) for the specified channel
//		Default value is 5 ms.
// Note:
//		The requested period, not the coerced period, is returned
//		See PS_SetPeriod and PS_GetStimPatternDuration
//
PLEXSTIM_API int       PS_GetPeriod(int StimN, int ChN, double *period);


// PS_GetStimPatternDuration - Get duration of the whole stimulation pattern (in s)
// In: 
//      StimN		- stimulator number to query (starts from 1)
//      ChN			- channel number to query (starts from 1) 
//		*duration	- pointer to a variable to store the pattern duration (in s)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
//		duration is updated with the stimulation pattern duration for the specified channel
// Effect:
//		Gets the duration in s of the whole stimulation pattern for the specified channel
//		The duration is the (coerced) period of repetition times the number of repetitions. 
//		See PS_SetPeriod or PS_SetRate for coersion
// Note:
//		Since the (coerced) period of repetition is always longer than the length of the pulse/waveform,
//		this function overestimates the duration of the stimulation pattern by an amount equal to
//		the difference between the (coerced) period and the length of the pulse/waveform
//
PLEXSTIM_API int       PS_GetStimPatternDuration(int StimN, int ChN, double *duration);


// PS_IsWaveformBalanced - Check the stimulation pattern to see if it is charge balanced.
// In: 
//		StimN		- stimulator number to query (starts from 1)
//		ChN			- channel number to query (starts from 1) 
//		*balanced	- pointer to a variable to store the charge balance indicator
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
//		balanced is set to true if the waveform is balanced (net charge is zero), false otherwise
// Effect:
//		Checks if the requested stimulation waveform is balanced
// ** WARNING **
//		Stimulation with waveforms that are NOT charge balanced can lead to electrode and tissue
//		damage. It is highly recommended to use a charge balanced waveform and to verify that 
//		the waveform as programmed is charge balanced.
//
PLEXSTIM_API int       PS_IsWaveformBalanced(int StimN, int ChN, bool *balanced);


// PS_SetTriggerMode - Set trigger mode for specified stimulator 
// In: 
//		StimN	- stimulator number to configure (starts from 1)
//		mode	- PS_TRIG_MODE variable (defined in PlexStimTypes.h)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
// Effect:
//		Sets the trigger mode for the specified stimulator (applies to all channels)
//      mode = PS_TRIG_SOFT  -	Stimulation can only be started by a software command (default);
//		mode = PS_TRIG_PULSE -	Stimulation can only be started by a digital input.
//								Stimulation begins when the digital input for the channel transitions from low (~0V) to high (~5V);
//		mode = PS_TRIG_LEVEL -  Stimulation can only be started by a digital input.
//								Stimulation begins when the digital input transitions from low (~0V) to high (~5V), 
//								If the digital input is still high when the stimulation protocol completes then 
//								the stimulation protocol will begin again.
//
PLEXSTIM_API int       PS_SetTriggerMode(int StimN, PS_TRIG_MODE mode);


// PS_GetTriggerMode - Get trigger mode for specified stimulator 
// In:
//		StimN	- stimulator number to query (starts from 1)
//		*mode	- pointer to a variable to store the trigger mode (type PS_TRIG_MODE, defined in PlexStimTypes.h)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
//		 mode is updated with the trigger mode for the specified stimulator 
// Effect:
//		Gets the trigger mode for the specified channel (see PS_SetTriggerMode for a description of the different modes) 
//
PLEXSTIM_API int       PS_GetTriggerMode(int StimN, PS_TRIG_MODE *mode);


// PS_SetDigitalOutputMode - Set the digital output mode for the specified stimulator
// In: 
//		StimN				- stimulator number to configure (starts from 1)
//		mode				- the desired mode (type PS_DIGITAL_OUTPUT, PS_DIGITAL_OUTPUT_HIGH or PS_DIGITAL_OUTPUT_LOW - defined in PlexStimTypes.h)
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
// Effect:
//		Each channel has a digital output that indicates when stimulation is occurring on that channel.
//		The digital output is always high during a pulse / arbitrary waveform output. 
//		When multiple repetitions of the pulse / waveform are configured, the digital output mode determines whether 
//		the digital output is high or low during the period in between pulses / arbitrary waveform repetitions.
//		Default = PS_DIGITAL_OUTPUT_LOW
//
PLEXSTIM_API int       PS_SetDigitalOutputMode(int StimN, PS_DIGITAL_OUTPUT mode);


// PS_GetDigitalOutputMode - Checks how the Digital Output mode is set
// In: 
//		StimN	- stimulator number to query (starts from 1) 
//		*mode	- pointer to a variable to store the digital output mode
//Returns:
//		-1 - invalid argument(s)
//		 0 - OK;
//		 Mode is updated with the digital output mode for the selected stimulator (type PS_DIGITAL_OUTPUT, defined in PlexStimTypes.h
// Effect:
//		Gets the digital output mode setting of the specified stimulator
//		See PS_SetDigitalOutputMode for a description of the modes.
//
PLEXSTIM_API int       PS_GetDigitalOutputMode(int StimN, PS_DIGITAL_OUTPUT *mode);


// PS_GetFwVersion - Get Firmware Version number
// In: 
//		StimN		- stimulator number to query (starts from 1) 
//		*fw_version	- pointer to a variable to store the firmware version number
// Returns:
//		0 - OK
//		1 - device error
//		fw_version is updated with the firmware version of the specified stimulator.
// Effect:
//		Gets the firmware version number of the selected stimulator.
//	    This version of the PlexStim was designed for use with firmware version 6.
//
PLEXSTIM_API int PS_GetFwVersion(int StimN, int *fw_version);


// PS_SetAutoDischarge - Enables/disables automatic discharge. ** SEE WARNING BELOW **
// In: 
//		StimN	- stimulator number to configure (starts from 1)
//		Enabled	- TRUE to enable automatic discharge; FALSE to disable automatic discharge
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
// Effect:
//		Enables/disables automatic discharge. Default = ENABLED.
//
// ** WARNING **
//		Disabling automatic discharge is ONLY recommended in very specific circustances
//		when the stimulator is used with the AStAR system. READ THE SYSTEM MANUALS.
//
PLEXSTIM_API int PS_SetAutoDischarge(int StimN, bool Enabled);


// PS_GetAutoDischarge - Checks automatic discharge setting. ** SEE WARNING BELOW ** 
// In: 
//		StimN		- stimulator number to query (starts from 1)
//		*bDischarge - pointer to a variable to store the auto discharge setting
// Returns:
//		-1 - invalid argument(s)
//		 0 - OK
//		 1 - device error
//		bDischarge is updated with the discharge mode of the specified stimulator
// Effect:
//		Checks if automatic discharge is enabled or disabled. Default = ENABLED.
//
// ** WARNING **
//		Disabling automatic discharge is ONLY recommended in very specific circustances
//		when the stimulator is used with the AStAR system. READ THE SYSTEM MANUALS.
//
PLEXSTIM_API int PS_GetAutoDischarge(int StimN, bool *bDischarge);


// PS_GetSerialNumber - Returns the serial number of the stimulator
// In: 
//		StimN	- stimulator number to query (starts from 1)
//		*serial	- pointer to a character array to store the serial number
// Returns:
//		0 - OK
//		1 - device error
//		14	- character array is not initialized (the pointer is null)
//		serial is updated with the serial number of the specified stimulator
// Effect:
//		Reads back the serial number of the Stimulator
// Note:
//		A buffer of at least 16 bytes must be allocated before calling this function
//		example C/C++ code:
//		char serial_number[16];
//		int res = PS_GetSerialNumber(StimN, serial_number);
//		
PLEXSTIM_API int PS_GetSerialNumber(int StimN, char *serial);


// PS_GetDescription - Returns the description (hardware model) of the specified stimulator
// In: 
//		StimN			- stimulator number to query (starts from 1)
//		*description	- pointer to a character array to store the description
// Returns:
//		0 - OK
//		1 - device error
//		14	- character array is not initialized (the pointer is null)
//		description is updated with the description (hardware model) of the specified stimulator
// Effect:
//		Reads the description (hardware model) of the specified stimulator.
// Note:
//		A buffer of at least 64 bytes must be allocated before calling this function
//		example C/C++ code:
//		char desc[64];
//		int res = PS_GetDescription(StimN, desc);
//		
PLEXSTIM_API int PS_GetDescription(int StimN, char *description);