////////////////////////////////////////
//                                    //
//    Plexon ElectroStim API          //
//                                    //
////////////////////////////////////////
//                                    //
//    Copyright 2015 Plexon Inc       //
//    Dallas Texas USA                //
//                                    //
//    Ver. 2.2.0.0  July 2014  TVM    //
//    Ver. 2.3.17.0	April 2015 A.E.   //
//                                    //
////////////////////////////////////////

#ifndef StimTypes_h
#define StimTypes_h
//---------------------------------------------------------------------------

//types of patterns to use by stimulator
//used in PS_SetPatternType and PS_GetPatternType

typedef enum {
	PS_PATTERN_RECT,	//rectangular pulse
	PS_PATTERN_ARB		//arbitrary waveform pattern
} PS_PATTERN_TYPE;



//trigger modes to use by stimulator
//used in PS_SetTriggerMode and PS_GetTriggerMode

typedef enum {
	PS_TRIG_SOFT,	  //stimulation will be started from software
	PS_TRIG_PULSE,	//stimulation will begin when the digital inplace for the channel transitions from low (~0V) to high (~5V)
	PS_TRIG_LEVEL	//stimulation begins when the digital inplace transitions from low (~0V) to high (~5V), 
					        //but if the digital inplace is still high when the stimulation protocol completes then 
					        //the stimulation protocol will begin again
} PS_TRIG_MODE;

typedef enum {
	PS_VMON_SCALING_0_25 = 0,	//0.25 V/V
	PS_VMON_SCALING_2_5,	    //2.5 V/V
	PS_VMON_SCALING_25,		    //25 V/V
	PS_VMON_SCALING_250  	    //250 V/V
} PS_VMON_SCALING;


typedef enum {
  PS_DIGITAL_OUTPUT_HIGH, 
  PS_DIGITAL_OUTPUT_LOW
} PS_DIGITAL_OUTPUT;

namespace PS_STATUS{
  enum{
    PS_INVALID_ARGUMENTS      = -1,
    PS_OK                     = 0, 
    PS_DEVICE_ERROR           = 1,
    PS_NO_PLEXON_STIM         = 2,
    PS_CRC_ERROR              = 3,
    PS_WRONG_TRIG_MODE        = 4,
    PS_MISMATCH_NPOINTS       = 5,
    PS_PATTERN_NOT_READY      = 6,
    PS_INVALID_FILE_NAME      = 7,
    PS_FILE_NOT_FOUND         = 8,
    PS_FILE_OPEN_FAILED       = 9,
    PS_FILE_TOO_MANY_LINES    = 10,
    PS_FILE_INVALID_CONTENT   = 11,
	PS_FILE_TOO_FEW_LINES     = 12,
	PS_FILE_MISMATCHED_PAIRS  = 13,
	PS_NULL_POINTER			  = 14
  };
};

//---------------------------------------------------------------------------
//parameters of the rectangular pulse
//used in PS_SetRectParam and PS_GetRectParam

struct PS_RectPattern {
	int Amp1;	//first phase amplitude, can range from +1000 uA to -1000 uA in steps of 1 uA
	int Amp2;	//second phase amplitude, can range from +1000 uA to -1000 uA in steps of 1 uA
	int W1;		//first phase width, range from 5 us to 65535 us in steps of 1 us
	int W2;		//second phase width, range from 5 us to 65535 us in steps of 1 us
	int Delay;	//interphase delay, can range from 5 us to 65535 us in steps of 1 us
};
//---------------------------------------------------------------------------
#endif